@@ -1,19 +1,49 @@
-# $Id: DVDRip.pm,v 1.55 2002/04/06 10:26:47 joern Exp $
+# $Id: DVDRip.pm,v 1.91 2002/11/01 16:22:13 joern Exp $
 
 package Video::DVDRip;
 
-$VERSION = "0.39_01";
+$VERSION = "0.47_09";
 
 use Carp;
 use FileHandle;
 
+%Video::DVDRip::deinterlace_filters = (
+	0          => "No deinterlacing",
+	1          => "Interpolate scanlines (fast)",
+	2          => "Handled by encoder (may segfault)",
+	3          => "Zoom to full frame (slow)",
+	'32detect' => "Automatic deinterlacing of single frames",
+);
+
+%Video::DVDRip::antialias_filters = (
+	0 => "No antialiasing",
+	1 => "Process de-interlace effects",
+	2 => "Process resize effects",
+	3 => "Process full frame (slow)",
+);
+
+%Video::DVDRip::audio_filters = (
+	'rescale'   => "None, volume rescale only",
+	'a52drc'    => "Range compression (liba52 filter)",
+	'normalize' => "Normalizing (mplayer filter)",
+);
+	
+
 init: {
+	# Skip transcode check during "make test". This makes
+	# automatic CPAN testing fail erroneously and makes
+	# problems during first dvd::rip install, when the
+	# use has no "." in PATH, because then dr_splitpipe isn't
+	# found.
+	last if $ENV{PERL_DL_NONLAZY} == 1;
+
 	my @path = split(":", $ENV{PATH});
+
 	my @programs = qw (
-		rm convert identify
+		rm convert identify pstree
 		transcode tcscan tccat
-		tcextract tcdecode splitpipe
-		
+		tcextract tcdecode
+		dr_splitpipe dr_progress
 	);
 	
 	my $missing = "";
@@ -37,23 +67,43 @@ init: {
 	my $ver = <$fh>;
 	close $fh;
 
-	$ver =~ m/v(\d+)\.(\d+)\.(\d+)/;
+	$ver =~ m/v(\d+)\.(\d+)\.(\d+)(.\d+)?/;
 	
-	# -------------------------
+	# ------------------------------
 	# transcode version numbers:
-	# -------------------------
+	# ------------------------------
 	# 0.5.3    => 503
 	# 0.6.0    => 600
 	# 1.2.7    => 100207
 	# 99.99.99 => 999999
-	# -------------------------
+	#
+	# 0.6.2.20021011 => 601.20021011
+	# ------------------------------
+
+	$TC::ORIG_VERSION = "$1.$2.$3$4";
 
 	$TC::VERSION = $1*10000+$2*100+$3;
 	$TC::VERSION ||= 0;
 
-	1;
+	if ( $4 ) {
+		# If we have a snapshot release, reduce the
+		# version number by 1 and add the snapshot
+		# date as decimals. This way version
+		# 6.0.2.20021011 is newer than 0.6.1 but
+		# older than 0.6.2 (final).
+
+		$TC::VERSION = $TC::VERSION - 1 + $4;
+	}
+
+	if ( $TC::VERSION < 600 ) {
+		print "Sorry, transcode versions prior 0.6.0 are no longer supported.\n".
+		      "Please upgrade transcode.\n";
+		exit 1;
+	}
 }
 
+1;
+
 __END__
 
 =head1 NAME
