@@ -1,4 +1,4 @@
-# $Id: Node.pm,v 1.15 2002/04/06 10:15:26 joern Exp $
+# $Id: Node.pm,v 1.20 2002/11/01 16:13:09 joern Exp $
 
 #-----------------------------------------------------------------------
 # Copyright (C) 2001-2002 Jrn Reder <joern@zyn.de> All Rights Reserved
@@ -28,9 +28,10 @@ sub username			{ shift->{username}			}
 sub ssh_cmd			{ shift->{ssh_cmd}			}
 sub speed			{ shift->{speed}			}
 sub tc_options			{ shift->{tc_options}			}
+sub answered_last_ping		{ shift->{answered_last_ping}		}
 
-sub progress_frames		{ shift->{progress_frames}		}
-sub progress_frames_cnt		{ shift->{progress_frames_cnt}		}
+sub progress_cnt		{ shift->{progress_cnt}			}
+sub progress_max		{ shift->{progress_max}			}
 sub progress_merge		{ shift->{progress_merge}		}
 sub progress_start_time		{ shift->{progress_start_time}		}
 sub assigned_job		{ shift->{assigned_job} 		}
@@ -47,9 +48,10 @@ sub set_username		{ shift->{username}		= $_[1] }
 sub set_ssh_cmd			{ shift->{ssh_cmd}		= $_[1] }
 sub set_speed			{ shift->{speed}		= $_[1] }
 sub set_tc_options		{ shift->{tc_options}		= $_[1] }
+sub set_answered_last_ping	{ shift->{answered_last_ping}	= $_[1]	}
 
-sub set_progress_frames		{ shift->{progress_frames}	= $_[1] }
-sub set_progress_frames_cnt	{ shift->{progress_frames_cnt}	= $_[1] }
+sub set_progress_cnt		{ shift->{progress_cnt}		= $_[1] }
+sub set_progress_max		{ shift->{progress_max}		= $_[1] }
 sub set_progress_merge		{ shift->{progress_merge}	= $_[1] }
 sub set_progress_start_time	{ shift->{progress_start_time}	= $_[1] }
 sub set_assigned_job		{ shift->{assigned_job} 	= $_[1] }
@@ -190,7 +192,7 @@ sub get_popen_code {
 
 	my $username = $self->username;
 	my $name     = $self->hostname;
-	my $ssh_cmd  = $self->ssh_cmd  || 'ssh -C';
+	my $ssh_cmd  = $self->ssh_cmd  || 'ssh -x';
 
 	$command =~ s/"/\\"/g;
 	$command = qq{$ssh_cmd $username\@$name "$command"};
@@ -201,6 +203,8 @@ sub get_popen_code {
 sub reset {
 	my $self = shift;
 	
+	$self->set_alive(0);
+	$self->set_answered_last_ping(0);
 	$self->set_state ('unknown')
 		if $self->state ne 'stopped' and
 		   $self->state ne 'aborted';
@@ -223,7 +227,7 @@ sub stop {
 	$self->log ("Node '".$self->name."' stopped");
 	$self->save;
 
-	$job->stop if $job;
+	$job->cancel if $job;
 	
 	1;
 }
@@ -295,7 +299,7 @@ sub run_tests {
 			);
 			$self->set_test_finished (1);
 		}
-	);
+	)->open;
 
 	1;
 }
@@ -310,7 +314,7 @@ sub get_test_command {
 	# 1. confirm ssh connection
 	$command .= "echo --ssh_connect-- 2>&1; ".
 		    "echo Ok 2>&1; ".
-		    "echo --ssh_connect-- 2>&1";
+		    "echo --ssh_connect-- 2>&1;";
 	
 	# 2. get content of data_base_dir
 	$data_base_dir ||= $self->data_base_dir;
@@ -326,9 +330,9 @@ sub get_test_command {
 		    "echo --write_test-- 2>&1; ";
 
 	# 4. get transcode version
-	$command .= "echo --transcode_version-- 2>&1; ".
-		    "transcode -h 2>&1; ".
-		    "echo --transcode_version-- 2>&1; ";
+	$command .= "echo --transcode_version--; ".
+		    "transcode -v 2>&1; ".
+		    "echo --transcode_version--; ";
 
 	$command .= "'";
 
@@ -342,12 +346,14 @@ sub parse_test_output {
 
 	# parse output
 	my %result;
+	$result{output} = $output;
 	foreach my $case ( qw ( ssh_connect data_base_dir_content
 				write_test transcode_version ) ) {
-		($result{$case}) = $output =~ m/--$case--\n(.*?)--$case--/s;
+		$output =~ s/--$case--\n(.*?)--$case--//s;
+		$result{$case} = $1;
 	}
 
-	$result{output} = $output;
+	$result{output_rest} = $output;
 
 	return \%result;
 }
