@@ -1,4 +1,4 @@
-# $Id: TitleTab.pm,v 1.31 2002/04/06 10:16:17 joern Exp $
+# $Id: TitleTab.pm,v 1.47 2002/11/01 13:31:30 joern Exp $
 
 #-----------------------------------------------------------------------
 # Copyright (C) 2001-2002 Jrn Reder <joern@zyn.de> All Rights Reserved
@@ -20,6 +20,8 @@ sub set_clist_row2title_nr	{ shift->{clist_row2title_nr}	= $_[1] }
 sub rip_title_widgets		{ shift->{rip_title_widgets}		}
 sub set_rip_title_widgets	{ shift->{rip_title_widgets}	= $_[1] }
 
+sub in_title_init		{ shift->{in_title_init}		}
+sub set_in_title_init		{ shift->{in_title_init}	= $_[1]	}
 
 #------------------------------------------------------------------------
 # Build RIP Title Tab
@@ -30,6 +32,8 @@ sub create_title_tab {
 
 	$self->set_rip_title_widgets({});
 
+	my $hsep;
+
 	my $vbox = Gtk::VBox->new;
 	$vbox->set_border_width(5);
 	$vbox->show;
@@ -42,10 +46,12 @@ sub create_title_tab {
 	$hbox->set_border_width(5);
 	$hbox->show;
 
-	my $button = Gtk::Button->new_with_label ("Read DVD Table of Contents");
+	my $button = Gtk::Button->new_with_label ("Read DVD table of contents");
 	$button->show;
 	$button->signal_connect ("clicked", sub { $self->read_dvd_toc } );
 
+	$self->rip_title_widgets->{read_dvd_toc_button} = $button;
+
 	my $label = Gtk::Label->new ("Press button, if list is empty or disc has changed.");
 	$label->show;
 
@@ -56,7 +62,7 @@ sub create_title_tab {
 	$vbox->pack_start ( $frame, 0, 1, 0);
 
 	# 2. TOC List / Frame
-	$frame = Gtk::Frame->new ("DVD Table of Contents");
+	$frame = Gtk::Frame->new ("DVD table of contents");
 	$frame->show;
 	$vbox->pack_start ( $frame, 0, 1, 0);
 
@@ -74,12 +80,19 @@ sub create_title_tab {
 	$sw->set_policy( 'automatic', 'automatic' );
 
 	my $clist = Gtk::CList->new_with_titles (
-		"Title", "Technical Information"
+		"Title", "Technical information"
 	);
 	$clist->show,
 	$clist->set_usize (450, 372);
-	$clist->set_selection_mode( 'browse' ); 
-	$clist->signal_connect ("select_row", sub { $self->cb_select_title (@_) } );
+	$clist->set_selection_mode( 'extended' ); 
+	$clist->signal_connect ("select_row",   sub {
+		return 1 if $self->in_title_init;
+		$self->cb_select_title (@_);
+	} );
+	$clist->signal_connect ("unselect_row", sub {
+		return 1 if $self->in_title_init;
+		$self->cb_select_title (@_);
+	} );
 
 	$sw->add( $clist );
 
@@ -94,7 +107,12 @@ sub create_title_tab {
 	
 	my $label_hbox = Gtk::HBox->new;
 	$label_hbox->show;
-	$label = Gtk::Label->new ("Select Audio Channel");
+	$label = Gtk::Label->new (
+		"Select audio track for volume\n".
+		"scanning.\n".
+		"(Does not affect ripping, all\n".
+		"audio tracks are ripped)"
+	);
 	$label->show;
 	$label->set_justify('left');
 	$label_hbox->pack_start ($label, 0, 1, 0);
@@ -113,10 +131,18 @@ sub create_title_tab {
 
 	$audio_vbox->pack_start($audio_popup, 0, 1, 0);
 
+	$hsep = Gtk::HSeparator->new;
+	$hsep->show;
+	$audio_vbox->pack_start($hsep, 0, 1, 0);
+
 	# Viewing Angle Selection
 	$label_hbox = Gtk::HBox->new;
 	$label_hbox->show;
-	$label = Gtk::Label->new ("Select Viewing Angle");
+	$label = Gtk::Label->new (
+		"Select viewing angle\n".
+		"(You must rip again if you\n".
+		"change this)"
+	);
 	$label->show;
 	$label->set_justify('left');
 	$label_hbox->pack_start ($label, 0, 1, 0);
@@ -135,10 +161,14 @@ sub create_title_tab {
 
 	$audio_vbox->pack_start($view_angle_popup, 0, 1, 0);
 
+	$hsep = Gtk::HSeparator->new;
+	$hsep->show;
+	$audio_vbox->pack_start($hsep, 0, 1, 0);
+
 	# Chapter mode ripping
 	$label_hbox = Gtk::HBox->new;
 	$label_hbox->show;
-	$label = Gtk::Label->new ("Specify Chapter Mode");
+	$label = Gtk::Label->new ("Specify chapter mode");
 	$label->show;
 	$label_hbox->pack_start ($label, 0, 1, 0);
 	$audio_vbox->pack_start ($label_hbox, 0, 1, 0);
@@ -163,7 +193,7 @@ sub create_title_tab {
 	# chapter selection list
 	$sw = new Gtk::ScrolledWindow( undef, undef );
 	$sw->set_policy( 'automatic', 'automatic' );
-	$sw->set_usize(undef, 156);
+	$sw->set_usize(undef, 138);
 
 	my $chapter_clist = Gtk::CList->new_with_titles ( "Chapter Selection" );
 	$sw->add( $chapter_clist );
@@ -183,15 +213,17 @@ sub create_title_tab {
 	$list_vbox->pack_start($hbox, 1, 1, 0);
 
 	my $button;
-	$button = Gtk::Button->new_with_label ("View Selected Title/Chapter(s)");
+	$button = Gtk::Button->new_with_label ("View\nselected title/chapter(s)");
 	$button->show;
 	$hbox->pack_start ($button, 1, 1, 0);
 	$button->signal_connect ("clicked",
 		sub { $self->view_title }
 	);
 
+	$self->rip_title_widgets->{view_title_button} = $button;
+
 	$button = Gtk::Button->new_with_label (
-		"RIP Selected Title/Chapter(s)"
+		"RIP\nselected title(s)/chapter(s)"
 	);
 	$button->show;
 	$button->signal_connect ("clicked",
@@ -199,21 +231,32 @@ sub create_title_tab {
 	);
 	$hbox->pack_start ($button, 1, 1, 0);
 
+	$self->rip_title_widgets->{rip_button} = $button;
+
 	# 6. Fill Content List, if we have content
 	$self->fill_content_list;
 
-	$self->rip_title_widgets->{tc_use_chapter_mode_all}->signal_connect (
-		"clicked", sub {
-			return 1 if not $self->selected_title;
-			$self->selected_title->set_tc_use_chapter_mode('all');
-			$self->init_chapter_list ( without_radio => 1 );
-		}
-	);
 	$self->rip_title_widgets->{tc_use_chapter_mode_no}->signal_connect (
 		"clicked", sub {
 			return 1 if not $self->selected_title;
 			$self->selected_title->set_tc_use_chapter_mode(0);
 			$self->init_chapter_list ( without_radio => 1 );
+			$self->rip_title_widgets->{rip_button}->set_sensitive(1);
+			$self->rip_title_widgets->{rip_button}->set_sensitive(1);
+			$self->set_render_vobsub_sensitive;
+			1;
+		}
+	);
+	$self->rip_title_widgets->{tc_use_chapter_mode_all}->signal_connect (
+		"clicked", sub {
+			return 1 if not $self->selected_title;
+			$self->selected_title->set_tc_use_chapter_mode('all');
+			$self->init_chapter_list ( without_radio => 1 );
+			$self->rip_title_widgets->{rip_button}->set_sensitive(
+				$self->project->rip_mode eq 'rip'
+			);
+			$self->set_render_vobsub_sensitive;
+			1;
 		}
 	);
 	$self->rip_title_widgets->{tc_use_chapter_mode_select}->signal_connect (
@@ -221,6 +264,11 @@ sub create_title_tab {
 			return 1 if not $self->selected_title;
 			$self->selected_title->set_tc_use_chapter_mode('select');
 			$self->init_chapter_list ( without_radio => 1 );
+			$self->rip_title_widgets->{rip_button}->set_sensitive(
+				$self->project->rip_mode eq 'rip'
+			);
+			$self->set_render_vobsub_sensitive;
+			1;
 		}
 	);
 	my $select_callback =  sub {
@@ -242,21 +290,36 @@ sub cb_select_title {
 	my $self = shift; $self->trace_in;
 	my ($clist, $row, $column, $event) = @_;
 
+	my @sel = $clist->selection;
+	if ( @sel > 1 ) {
+		$self->rip_title_widgets->{view_title_button}->set_sensitive(0);
+		return;
+	}
+	$self->rip_title_widgets->{view_title_button}->set_sensitive(1);
+	$row = $sel[0];
+
 	my $nr = $self->clist_row2title_nr->{$row};
 	$self->project->set_selected_title_nr ($nr);
-	$self->set_selected_title($self->project->content->titles->{$nr});
+	my $title = $self->set_selected_title($self->project->content->titles->{$nr});
 
-#	warn "remove me!!!";
-#	my $title = $self->selected_title;
-#	if ( $title ) {
-#		$title->analyze_probe_output (
-#			output => $title->probe_result->probe_output,
-#		);
-#		$title->probe_audio;
-#	}
-	
 	$self->fill_with_values;
 	
+	my $rip_mode = $self->project->rip_mode;
+	
+	if ( $rip_mode eq 'rip' ) {
+		$self->rip_title_widgets->{view_title_button}->set_sensitive(1);
+		$self->rip_title_widgets->{rip_button}->set_sensitive(1);
+
+	} elsif ( $rip_mode eq 'dvd' or $rip_mode eq 'dvd_image' ) {
+		$self->rip_title_widgets->{view_title_button}->set_sensitive(
+			$rip_mode eq 'dvd'
+		);
+		$self->rip_title_widgets->{rip_button}->set_sensitive(
+			not $title->tc_use_chapter_mode
+		) if $title;
+
+	}
+	
 	1;
 }
 
@@ -267,36 +330,15 @@ sub init_audio_popup {
 	my $title = $self->selected_title;
 	return if not $title;
 
-	my $audio_popup = $self->rip_title_widgets->{audio_popup};
-	
-	my $audio_popup_menu = Gtk::Menu->new;
-	$audio_popup_menu->show;
-	$audio_popup->set_menu($audio_popup_menu);
-
-	my $item;
-	my $i = 0;
-	foreach my $audio ( @{$title->audio_tracks} ) {
-		my $sample_rate = $audio->{sample_rate};
-		$sample_rate = "48kHz"   if $sample_rate == 48000;
-		$sample_rate = "41.1kHz" if $sample_rate == 44100;
-		$item = Gtk::MenuItem->new (
-			"$i: $audio->{lang} $audio->{type} ".
-			"$sample_rate $audio->{channels}Ch"
-		);
-		$item->show;
-		$item->signal_connect (
-			"select", sub {
-				$_[1]->set_audio_channel($_[2]);
-				$self->init_title_labels;
-				$self->init_transcode_values;
-			},
-			$title, $i
-		);
-		$audio_popup_menu->append($item);
-		++$i;
-	}
-
-	$audio_popup->set_history($title->audio_channel);
+	$self->fill_audio_popup (
+		audio_popup => $self->rip_title_widgets->{audio_popup},
+		tab         => "rip_title",
+	);
+	$self->fill_audio_popup (
+		audio_popup => $self->transcode_widgets->{select_audio_channel_popup},
+		tab         => "transcode",
+	);
+	$self->fill_target_audio_popup;
 
 	# viewing angle popup
 
@@ -306,6 +348,7 @@ sub init_audio_popup {
 	$view_angle_popup_menu->show;
 	$view_angle_popup->set_menu($view_angle_popup_menu);
 
+	my $item;
 	foreach my $angle ( 1 .. $title->viewing_angles ) {
 		$item = Gtk::MenuItem->new ( "Angle $angle" );
 		$item->show;
@@ -324,6 +367,186 @@ sub init_audio_popup {
 	1;
 }
 
+sub fill_audio_popup {
+	my $self = shift;
+	my %par = @_;
+	my ($audio_popup, $tab) = @par{'audio_popup','tab'};
+
+	$self->print_debug ("fill_audio_popup: entered for tab '$tab'");
+
+	my $title = $self->selected_title;
+
+	my $audio_popup_menu = Gtk::Menu->new;
+	$audio_popup_menu->show;
+	$audio_popup->remove_menu;
+	$audio_popup->set_menu($audio_popup_menu);
+
+	my $tc_audio_tracks = $title->tc_audio_tracks;
+
+	my $item;
+	my $i = 0;
+	my @items;
+	foreach my $audio ( @{$title->audio_tracks} ) {
+		my $sample_rate = $audio->sample_rate;
+		$sample_rate = "48kHz"   if $sample_rate == 48000;
+		$sample_rate = "41.1kHz" if $sample_rate == 44100;
+
+		my $target_track = "";
+		if ( $tc_audio_tracks->[$i]->tc_target_track != -1 ) {
+			$target_track = " => ".$tc_audio_tracks->[$i]->tc_target_track;
+		} else {
+			$target_track = " => skip";
+		}
+		
+		$target_track = "" if $tab eq 'rip_title';
+		
+		$item = Gtk::MenuItem->new (
+			"$i: ".$audio->lang." ".$audio->type." ".
+			"$sample_rate ".$audio->channels."Ch".
+			$target_track
+		);
+
+		$self->print_debug ("fill_audio_popup: New item: ".$item->child->get);
+
+		$item->show;
+		$item->signal_connect (
+			"select", sub {
+				return if $self->in_transcode_init;
+				$_[1]->set_audio_channel($_[2]);
+				$self->init_title_labels;
+				$self->configure_target_audio_popup;
+				if ( $tab eq 'rip_title' ) {
+					$self->init_audio_values (
+					    switch_popup =>
+						$self->transcode_widgets
+						     ->{select_audio_channel_popup}
+					);
+					$self->init_transcode_values (
+						no_audio => 1
+					);
+				} else {
+					$self->init_audio_values (
+					    switch_popup =>
+						$self->rip_title_widgets
+						     ->{audio_popup}
+					);
+					$self->init_transcode_values (
+						no_audio => 1
+					);
+				}
+				1;
+			},
+			$title, $i
+		);
+		$audio_popup_menu->append($item);
+		push @items, $item;
+		++$i;
+	}
+
+	if ( $title->audio_channel == -1 ) {
+		$item = Gtk::MenuItem->new ("No audio");
+		$item->show;
+		$audio_popup_menu->append($item);
+		$audio_popup->set_history(0);
+		
+	} else {
+		$audio_popup->set_history($title->audio_channel);
+	}
+
+	$self->transcode_widgets->{"items_$tab"} = \@items;
+
+	1;
+}
+
+sub configure_target_audio_popup {
+	my $self = shift;
+	
+	my $title = $self->selected_title;
+
+	my $items = $self->rip_title_widgets->{target_popup_items};
+	
+	my %track_is_assigned;
+	foreach my $audio ( @{$title->tc_audio_tracks} ) {
+		$track_is_assigned{$audio->tc_target_track} = 1
+			if $audio->tc_target_track != -1;
+	}
+
+	my $j = -1;
+	foreach my $it ( @{$items} ) {
+		$it->set_sensitive(
+			!($track_is_assigned{$j} and 
+			  $title->tc_target_track != $j)
+		);
+		++$j;
+	}
+
+	1;
+}
+
+sub fill_target_audio_popup {
+	my $self = shift;
+
+	my $title = $self->selected_title;
+
+	my $audio_popup = $self->transcode_widgets->{tc_target_audio_channel_popup};
+
+	my $audio_popup_menu = Gtk::Menu->new;
+	$audio_popup_menu->show;
+	$audio_popup->set_menu($audio_popup_menu);
+
+	my %track_is_assigned;
+	foreach my $audio ( @{$title->tc_audio_tracks} ) {
+		$track_is_assigned{$audio->tc_target_track} = 1
+			if $audio->tc_target_track != -1;
+	}
+
+	my @items;
+	my $item;
+	my $text;
+	my %history;
+	my $history = 0;
+	for (my $i=-1; $i < @{$title->audio_tracks}; ++$i ) {
+		$text = $i == -1 ? "Skip / Decativate this track" : "Track #$i";
+		$history{$i} = $history;
+		$item = Gtk::MenuItem->new ($text);
+		push @items, $item;
+		$item->show;
+		$item->set_sensitive(0) if $track_is_assigned{$i} and
+			$title->tc_target_track != $i;
+		$item->signal_connect (
+			"select", sub {
+				return if $self->in_transcode_init;
+				$title->set_tc_target_track ($_[1]);
+				$title->calc_video_bitrate;
+				$self->init_audio_values (
+					dont_set_target_popup => 1
+				);
+				# this corrects the target track
+				# in both popups
+				$self->fill_audio_popup (
+					audio_popup => $self->transcode_widgets
+						 	    ->{select_audio_channel_popup},
+					tab	    => "transcode"
+				);
+				$self->fill_audio_popup (
+					audio_popup => $self->rip_title_widgets
+						 	    ->{audio_popup},
+					tab	    => "rip_title"
+				);
+				$self->calc_video_bitrate;
+				1;
+			},$i,
+		);
+		$audio_popup_menu->append($item);
+		++$history;
+	}
+
+	$audio_popup->set_history ($history{$title->tc_target_track});
+	$self->rip_title_widgets->{target_popup_items} = \@items;
+
+	1;
+}
+
 sub init_chapter_list {
 	my $self = shift; $self->trace_in;
 	my %par = @_;
@@ -380,19 +603,19 @@ sub init_title_labels {
 			       ->audio_tracks
 			       ->[$audio_channel];
 		$audio_label =
-			"Viewing Angle #".$title->tc_viewing_angle.", ".
-			"Audio Channel: #$audio_channel - ".
-			"$audio->{lang} $audio->{type} ".
-			"$audio->{sample_rate} $audio->{channels}Ch";
+			"Viewing angle #".$title->tc_viewing_angle.", ".
+			"Audio track: #$audio_channel - ".
+			$audio->lang." ".$audio->type." ".
+			$audio->sample_rate." ".$audio->channels."Ch";
 	} else {
-		$audio_label = "No Audio\n";
+		$audio_label = "No audio\n";
 	}
 
 	my $nr = $title->nr;
 
 	foreach my $label ( @{$self->gtk_title_labels} ) {
 		$label->set_text (
-			"DVD Title #$nr - ".
+			"DVD title #$nr - ".
 			$self->get_title_info (title => $title).
 			"\n".$audio_label
 		);
@@ -414,11 +637,15 @@ sub read_dvd_toc {
 
 	$self->clear_content_list;
 
+	my $mounted = $project->dvd_is_mounted;
+	$project->mount_dvd if not $mounted;
+
 	# read TOC
 	eval {
 		$project->check_dvd_in_drive;
 		$content->read_title_listing;
 	};
+
 	if ( $@ ) {
 		$self->message_window (
 			message => "Can't read DVD TOC. Please put ".
@@ -426,76 +653,53 @@ sub read_dvd_toc {
 				   "Internal message was:\n".
 				   $self->stripped_exception
 		);
+		$project->umount_dvd if not $mounted;
 		return;
 	}
 	
 	my $titles = $content->get_titles_by_nr;
-	my $step = -1;
 
-	my $open_callback = sub {
-		++$step;
-		return $titles->[$step]->probe_async_start;
-	};	
-
-	my $progress_callback = sub {
-		return $step+1;
-	};
-
-	my $close_callback = sub {
-		my %par = @_;
-		my ($progress, $output) = @par{'progress','output'};
-
-# print "---gotoutput:$output\n---end-of-gotoutput\n\n";
-		eval {
-			$titles->[$step]->probe_async_stop (
-				fh     => $progress->fh,
-				output => $output
-			);
-			$titles->[$step]->suggest_transcode_options;
-		};
-
-		if ( not $@ ) {
-			$self->append_content_list ( title => $titles->[$step] );
-			$self->log ("Successfully probed title #".($step+1));
-
-		} else {
-			$self->message_window (
-				message => "Can't probe Track #$step\n\n".
-					   "Track will not be listed.\n\n".
-					   "Output of tcprobe was:\n\n$output\n\n$@"
-			);
-			$self->log ("Error probing title #".($step+1));
-		}
-		
-		++$step;
-
-		if ( $step == @{$titles} ) {
-			my $nr = $self->clist_row2title_nr->{0};
-			$self->project->set_selected_title_nr ($nr);
-			my $title = $self->project->content->titles->{$nr};
-			$self->set_selected_title( $title );
-			$self->fill_with_values;
-			return 'finished';
-
-		} else {
-			$progress->init_pipe (
-				fh => $titles->[$step]->probe_async_start
-			);
-			return 'continue';
-		}
-	};
-
-	$self->comp('progress')->open (
-		label             => "Reading DVD TOC",
-		need_output       => 1,
-		show_eta          => 1,
-		show_percent      => 1,
-		show_fps          => 0,
-		max_value         => scalar(@{$titles}),
-		open_callback     => $open_callback,
-		progress_callback => $progress_callback,
-		close_callback    => $close_callback,
+	my $nr;
+	my $job;
+	my $last_job;
+	my $exec = Video::DVDRip::GUI::ExecuteJobs->new (
+		reuse_progress => 1
 	);
+	
+	foreach my $title ( @{$titles} ) {
+		$job  = Video::DVDRip::Job::Probe->new (
+			nr    => ++$nr,
+			title => $title,
+		);
+
+		$job->set_progress_max(scalar(@{$titles})+0.001);
+
+		$job->set_cb_finished (sub {
+			$self->append_content_list ( title => $title );
+		});
+
+		$last_job = $exec->add_job ( job => $job );
+	}
+	
+	$exec->set_cb_finished (sub{
+		$project->umount_dvd if not $mounted;
+
+		return if $exec->cancelled;
+
+		my $nr = $self->clist_row2title_nr->{0};
+		$self->project->set_selected_title_nr ($nr);
+
+		my $title = $self->project->content->titles->{$nr};
+		$self->set_selected_title( $title );
+		$self->fill_with_values;
+		$self->rip_title_widgets->{content_clist}->select_row (0,0);
+		$self->project->copy_ifo_files;
+		$self->project->backup_copy;
+
+		1;
+	});
+
+	$exec->execute_jobs;
 
 	1;
 }
@@ -532,6 +736,12 @@ sub fill_content_list {
 	     ->{content_clist}
 	     ->select_row ($select_row, 1);
 
+	if ( $self->project->rip_mode ne 'rip' ) {
+		$self->rip_title_widgets->{rip_button}->set_sensitive(0);
+	} else {
+		$self->rip_title_widgets->{rip_button}->set_sensitive(1);
+	}
+
 	1;
 }
 
@@ -542,7 +752,6 @@ sub append_content_list {
 
 	my $row = $self->rip_title_widgets->{content_clist}->append (
 		$title->nr,
-#		int($title->size/1024/1024),
 		$self->get_title_info ( title => $title ),
 	);
 	
@@ -578,7 +787,7 @@ sub get_title_info {
 sub create_selected_title {
 	my $self = shift; $self->trace_in;
 
-	my $frame = Gtk::Frame->new ("Selected Title");
+	my $frame = Gtk::Frame->new ("Selected DVD title");
 	$frame->show;
 
 	my $hbox = Gtk::HBox->new;
@@ -598,187 +807,104 @@ sub create_selected_title {
 	return $frame;
 }
 
-sub rip_title {
-	my $self = shift; $self->trace_in;
+sub rip_title_selection_sensitive {
+	my $self = shift;
+	my ($value) = @_;
 
-	my $title = $self->selected_title;
-	return if not $title;
-	return if $self->comp('progress')->is_active;
-
-	eval { $self->project->check_dvd_in_drive };
-	if ( $@ ) {
-		$self->message_window (
-			message => "Please put a disc into your drive."
-		);
-		return;
-	}
-	$self->project->check_dvd_in_drive;
-
-	return $self->rip_title_chapters if $title->tc_use_chapter_mode;
-
-	my $with_scanning = $title->audio_channel != -1;
-
-	my $start_method = $with_scanning ? "rip_and_scan_async_start" :
-					    "rip_async_start";
-	my $stop_method  = $with_scanning ? "rip_and_scan_async_stop" :
-					    "rip_async_stop";
-
-	my $open_callback = sub {
-		return $title->$start_method();
-	};
-
-	my $frames = 0;
-	my $progress_callback = sub {
-		# no progress bar for versions prior 0.6.0(pre)
-		return 1 if $TC::VERSION < 600;
-
-		# otherwise we get the output of "tcdemux -W" here,
-		# where we just need to count lines (one frame per line)
-		my %par = @_;
-		my ($buffer) = @par{'buffer'};
-		$frames += $buffer =~ tr/\n/\n/;
-		return $frames;
-	};
-
-	my $close_callback = sub {
-		my %par = @_;
-		my ($progress, $output) = @par{'progress', 'output'};
-		$title->$stop_method (
-			fh     => $progress->fh,
-			output => $output,
-		);
-		$title->probe_audio;
-		$title->suggest_transcode_options;
-		$self->fill_with_values;
-		$title->calc_program_stream_units if $TC::VERSION >= 600;
-		return 'finished';
-	};
-
-	my $cancel_callback = sub {
-		my %par = @_;
-		my ($progress) = @par{'progress'};
-		system ("killall splitpipe");
-		close ($progress->fh);
-		$title->remove_vob_files;
-		return 1;
-	};
-
-	$self->comp('progress')->open (
-		label             => "Ripping Title #".$title->nr,
-		need_output       => 1,
-		show_percent      => ($TC::VERSION >= 600),
-		show_eta          => ($TC::VERSION >= 600),
-		show_fps          => ($TC::VERSION >= 600),
-		max_value         => ($TC::VERSION >= 600 ? $title->frames : 1),
-		open_callback     => $open_callback,
-		progress_callback => $progress_callback,
-		cancel_callback   => $cancel_callback,
-		close_callback    => $close_callback,
-	);
+	my $widgets = $self->rip_title_widgets;
+	
+	$widgets->{content_clist}              -> set_sensitive($value);
+	$widgets->{audio_popup}                -> set_sensitive($value);
+	$widgets->{view_angle_popup}           -> set_sensitive($value);
+	$widgets->{tc_use_chapter_mode_all}    -> set_sensitive($value);
+	$widgets->{tc_use_chapter_mode_no}     -> set_sensitive($value);
+	$widgets->{tc_use_chapter_mode_select} -> set_sensitive($value);
+	$widgets->{chapter_select_clist}       -> set_sensitive($value);
+	$widgets->{rip_button}                 -> set_sensitive($value);
+	$widgets->{read_dvd_toc_button}        -> set_sensitive($value);
 
 	1;
 }
 
-sub rip_title_chapters {
+sub rip_title {
 	my $self = shift; $self->trace_in;
-	
-	my $title = $self->selected_title;
-	return if not $title;
+
 	return if $self->comp('progress')->is_active;
 
-	my $chapter_mode = $title->tc_use_chapter_mode;
-
-	croak "Title is not in chapter mode" if not $chapter_mode;
-
-	my $nr = $title->nr;
-
-	my @chapters  = @{$title->get_chapters};
-	my $max_value = $chapter_mode eq 'select' ?
-				@chapters : 
-				int ($title->size / 1024);
-
-	if ( not @chapters ) {
+	if ( not $self->selected_title ) {
 		$self->message_window (
-			message => "No chapters selected."
+			message => "Please select at least one title."
 		);
 		return;
 	}
 
-	my $cnt = 1;
-	my $base_progress = 0;
-	my $old_progress  = 0;
+	$self->rip_title_selection_sensitive(0);
 
-	my $chapter = shift @chapters;
+	my $project = $self->comp('project')->project;
 
-	$title->set_actual_chapter ( $chapter );
+	my $mounted = $project->dvd_is_mounted;
+	$project->mount_dvd if not $mounted;
 
-	my $open_callback = sub {
-		return $title->rip_async_start;
-	};
-
-	my $progress_callback = sub {
-		return $cnt if $chapter_mode eq 'select';
-		my %par = @_;
-		my ($buffer) = @par{'buffer'};
-		$buffer =~ /(\d+)-(\d+)\n[^\n]*$/s;
-		my ($chunk, $bytes) = ($1, $2);
-		my $progress = $base_progress + ($chunk-1)*1024*1024 + int($bytes/1024);
-		$progress = $old_progress if $progress < $old_progress;
-		$old_progress = $progress if $progress > 0;
-		return $base_progress + ($chunk-1)*1024*1024 + int($bytes/1024);
-	};
-
-	my $close_callback = sub {
-		my %par = @_;
-		my ($progress, $output) = @par{'progress', 'output'};
-		$title->rip_async_stop (
-			fh     => $progress->fh,
-			output => $output,
+	eval { $self->project->check_dvd_in_drive };
+	if ( $@ ) {
+		$self->rip_title_selection_sensitive(1);
+		$self->message_window (
+			message => "Please put a disc into your drive. ($@)"
 		);
+		$project->umount_dvd if not $mounted;
+		return;
+	}
 
-		++$cnt;
-		$chapter = shift @chapters;
-		$title->set_actual_chapter($chapter);
+	my @sel = $self->rip_title_widgets->{content_clist}->selection;
+	
+	my $nr;
+	my $job;
+	my $last_job;
+	my $exec = Video::DVDRip::GUI::ExecuteJobs->new;
 
-		if ( not defined $chapter ) {
-			$title->suggest_transcode_options;
-			$self->fill_with_values;
-			return 'finished';
+	foreach my $sel ( @sel ) {
+		my $title = $self->project->content->titles->{$sel+1};
+
+		if ( not $title->tc_use_chapter_mode ) {
+		    my $job = Video::DVDRip::Job::Rip->new (
+			nr    => ++$nr,
+			title => $title,
+		    );
+		    $last_job = $exec->add_job ( job => $job );
 
 		} else {
-			$progress->set_label (
-				"Ripping Chapter $chapter of Title #".$title->nr
+		    foreach my $chapter ( @{$title->get_chapters} ) {
+			$job = Video::DVDRip::Job::Rip->new (
+			    nr    => ++$nr,
+			    title => $title,
 			);
-			$progress->init_pipe (
-				fh => $title->rip_async_start
-			);
-			$base_progress = $old_progress;
-			return 'continue';
+			$job->set_chapter ( $chapter );
+			$last_job = $exec->add_job ( job => $job );
+		    }
 		}
-	};
+	}
 
-	my $cancel_callback = sub {
-		my %par = @_;
-		my ($progress) = @par{'progress'};
-		close ($progress->fh);
-		$title->remove_vob_files;
-		$title->set_actual_chapter(undef);
-		return 1;
-	};
+	$exec->set_cb_finished (sub {
+		$project->umount_dvd if not $mounted;
 
-	$self->comp('progress')->open (
-		label             => "Ripping Chapter $chapter of Title #".$title->nr,
-		need_output       => 0,
-		show_percent      => ($chapter_mode ne 'select'),
-		show_eta          => ($chapter_mode ne 'select'),
-		show_fps          => 0,
-		max_value         => $max_value,
-		open_callback     => $open_callback,
-		progress_callback => $progress_callback,
-		cancel_callback   => $cancel_callback,
-		close_callback    => $close_callback,
+		if ( $exec->cancelled ) {
+			$exec->cancelled_job->title->remove_vob_files
+				if $exec->cancelled_job;
+		} else {
+			$self->grab_preview_frame;
+			$self->fill_with_values;
+			$self->project->backup_copy;
+		}
+
+		$self->rip_title_selection_sensitive(1);
+
+		1;
+	});
+
+	$exec->execute_jobs (
+		max_diskspace_needed => 6 * 1024
 	);
-	
+
 	1;
 }
 
@@ -786,12 +912,14 @@ sub view_title {
 	my $self = shift;
 
 	my $title = $self->selected_title;
-	return if not $title;
 
-	my $nr            = $title->nr;
-	my $audio_channel = $title->audio_channel;
-	
-	my @mrls;
+	if ( not $title ) {
+		$self->message_window (
+			message => "Please select a title."
+		);
+		return;
+	}
+
 	if ( $title->tc_use_chapter_mode eq 'select' ) {
 		my $chapters = $title->tc_selected_chapters;
 		if ( not $chapters or not @{$chapters} ) {
@@ -800,19 +928,13 @@ sub view_title {
 			);
 			return;
 		}
-		foreach my $i ( @{$chapters} ) {
-			push @mrls, "d4d://i".$nr."t0c".($i-1)."t".($i-1);
-		}
-	} else {
-		push @mrls, "d4d://i".$nr."t0c0t0";
 	}
+	
+	my $command = $title->get_view_dvd_command (
+		command_tmpl => $self->config('play_dvd_command')
+	);
 
-	my $command =
-		"xine ".
-		join (" ", @mrls).
-		" -a $audio_channel -p &";
-
-	system ($command);
+	system ($command." &");
 	
 	1;
 }
